/***************************************************************************
 *                                                                         *
 *   Zabit: Content and attachment filter for qmail.                       *
 *   Version 0.7.1                                                         *
 *   http://www.enderunix.org/zabit                                        *
 *                                                                         *
 *   Copyright (C) 2004 by N.Ersen SISECI                                  *
 *   siseci@enderunix.org                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include "zabit.h"
#include "scan.h"
#include "config.h"
#include "version.h"
#include "log.h"
#include "attach.h"

extern void setsyslocale();
extern void getsyslocale();
extern void setsysoldlocale();

int searchbinary = 0;

void usage(char *arg)
{
	printf(" Usage: %s [options]\n", arg);
	printf(" Available Options: \n");
	printf(" -b: binary file scan\n");
	printf(" -i: case-insensitive\n");
	printf(" -r: recursive scan\n");
	printf(" -f: zabit word file name\n");
	printf(" -d: scan directory\n");
	printf(" -a: zabit attach list file name\n");
	printf(" -t: zabit disable attach list file name\n");
	printf(" -l: log file\n");
	printf(" -s: slient (no log)\n");
	printf(" -o: scan only\n");
	printf(" -v: Version\n");
	printf(" -h: This help\n\n");
}

int main(int argc, char **argv)
{
	int i = 0;
	int f = 0, r = 0;
	int d = 0;
	int l = 0;
	int a = 0;
	int t = 0;
	int error = 0;

	int ch;
	extern char *optarg;
	extern int optind;
	char zabitconf[MAXFILENAMELEN];
	char zabitaconf[MAXFILENAMELEN];
	char zabitdaconf[MAXFILENAMELEN];
	char scandirname[MAXFILENAMELEN];
	struct stat statbuf;
	scanonly = 0;
	donotlog = 0;

	while ((!error) && (ch = getopt(argc, argv, "birhvosf:d:l:a:t:")) != -1) {
		switch (ch) {
		case 'v':
			printf("zabit Version %s\n", VERSION);
			exit(0);
		case 'h':
			usage(argv[0]);
			exit(0);
		case 'f':
			strncpy(zabitconf, optarg, MAXFILENAMELEN - 1);
			f = 1;
			break;
		case 'r':
			r = 1;
			break;
		case 'b':
			searchbinary = 1;
			break;
		case 'd':
			strncpy(scandirname, optarg, MAXFILENAMELEN - 1);
			d = 1;
			break;
		case 'i':
			caseinsensitive = 1;
			break;
		case 's':
			donotlog = 1;
			break;
		case 'o':
			scanonly = 1;
			break;
		case 'l':
			strncpy(log_file, optarg, MAXFILENAMELEN - 1);
			l = 1;
			break;
		case 'a':
			a = 1;
			strncpy(zabitaconf, optarg, MAXFILENAMELEN - 1);
			break;
		case 't':
			t = 1;
			strncpy(zabitdaconf, optarg, MAXFILENAMELEN - 1);
			break;
		default:
			usage(argv[0]);
			error = 1;
			exit(ERRORTMP);
		}
	}

	getsyslocale();
	setsyslocale();

	if (l == 0)
		sprintf(log_file, "/var/log/zabit.log");

	if (f == 1)
		readwordfile(zabitconf);
	else
		readwordfile(WORD_LIST);

	if (a == 1)
		readattachfile(zabitaconf);
	else
		readattachfile(ATTACH_FILE);

	if (t == 1)
		readdisableattachfile(zabitdaconf);
	else
		readdisableattachfile(DIS_ATTACH_FILE);

	if (d == 0) {
		i = scandirfilenames(".",r);
		if (i != NORMALMAIL)
			return i;
		else
			i = scanmaildir(".", r);
	}
	else {

		lstat(scandirname, &statbuf);
		if (S_ISDIR(statbuf.st_mode)) {
			i = scandirfilenames(scandirname,r);
			if (i != NORMALMAIL)
				return i;
			else
				i = scanmaildir(scandirname, r);
		}
		else if (S_ISREG(statbuf.st_mode)) {
			i = checkfilename(scandirname);
			if (i != NORMALMAIL)
				return i;
			else
				i = scanmailfile(scandirname, ".");
		}
		else
			return ERRORTMP;

	}
	setsysoldlocale();
	return i;
}
